unit Xmr_pgm;

{XMODEM/Receive}

{This program is meant to be a minimal outline of the}
{code necessary to receive a file using the XMODEM protocol.}

interface

uses
  wsc, xydrv, Display,
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Menus, StdCtrls;
const
  NAK = $15;
  XMODEM = 0;
type
  Txmr = class(TForm)
    MainMenu1: TMainMenu;
    mReceive: TMenuItem;
    mExit: TMenuItem;
    eResult: TMemo;
    procedure mExitClick(Sender: TObject);
    procedure mReceiveClick(Sender: TObject);
    procedure create(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    Port : Integer;
    BaudRate : LongInt;
    FileName : String;
  end;

var
  xmr: Txmr;

implementation

{$R *.DFM}

procedure Txmr.mExitClick(Sender: TObject);
begin
  SioDone(Port);
  Application.Terminate;
end;

procedure Txmr.mReceiveClick(Sender: TObject);

var
  I, Code : Integer;
  FilePtr : PChar;
  MsgPtr  : PChar;
  Packet  : Integer;
  LastPacket : Integer;
begin
  (* initialize WSC *)
  Code := SioReset(Port,2048,2048);
  if Code < 0 then
    begin
      DisplayLine(xmr.eResult,Format('Error %d: Cannot reset port',[Code]));
      DisplayError(xmr.eResult, Code);
      exit
    end;
  (* set parameters *)
  SioBaud(Port,BaudRate);
  SioParms(Port, WSC_NoParity, WSC_OneStopBit, WSC_WordLength8);
  SioDTR(Port,'S');
  SioRTS(Port,'S');
  (* set hardware flow control *)
  SioFlow(Port, 'H');
  (* acquire xyDriver *)
  xyAcquire(Port,Port);
  (* XM Receive *)
  GetMem(FilePtr,50);
  StrPCopy(FilePtr,FileName);
  DisplayLine(xmr.eResult,'Starting XMODEM receive');
  Code := xyStartRx(Port,FilePtr,CHR(NAK),XMODEM);
  FreeMem(FilePtr,50);
  (* check return code *)
  if Code < 0 then
    begin
      DisplayLine(xmr.eResult,Format('Error %d: XMODEM receive fails',[Code]));
      exit
    end;
  (* allocate memory for XY driver messages *)
  GetMem(MsgPtr,90);
  (* ready to go *)
  SCREEN.Cursor := crHourGlass;
  mReceive.Enabled := false;
  (* receive file *)
  while true do
    begin
      (* run the XY driver a few times *)
      for I:= 1 To 4 do Code := xyDriver(Port);
      while true do
        begin
          (* get next message from XY driver *)
          if xyGetMessage(Port,MsgPtr,90) <> 0 then
            DisplayLine(xmr.eResult,StrPas(MsgPtr))
          else break;
        end;
      (* is XY driver done ? *)
      if Code = XY_IDLE then
        begin
          (* XY driver is idle *)
          Code := xyGetParameter(Port,XY_GET_ERROR_CODE);
          if Code <> 0 then
            DisplayLine(xmr.eResult,Format('XY file transfer fails (%d)',[Code]))
          else DisplayLine(xmr.eResult,'File transfer complete');
          (* all done *)
          FreeMem(MsgPtr,90);
          xyRelease;
          SioDone(Port);
          mReceive.Enabled := true;
          SCREEN.Cursor := crDefault;
          exit
        end
      else
        begin
          (* XY driver is running *)
          Packet := xyGetParameter(Port,XY_GET_PACKET);
          if Packet <> LastPacket then
            begin
              DisplayLine(xmr.eResult,Format('Packet %d',[Packet]) );
              LastPacket := Packet
            end
        end
    end
end;

procedure Txmr.create(Sender: TObject);
begin
  (* PROGRAMMER: Edit next 3 lines to suite *)
  Port := COM2;
  BaudRate := 19200;
  FileName := 'file.bin';
  (* END-PROGRAMMER *)
  DisplayLine(xmr.eResult,Format('Port : COM%1d',[1+Port]));
  DisplayLine(xmr.eResult,Format('Baud : %d',[BaudRate]));
  DisplayLine(xmr.eResult,Format('File : %s',[FileName]));
  DisplayLine(xmr.eResult,'Push "Receive" above to receive file')
end;

end.
