unit Yms_pgm;

{YMODEM/Send}

{This program is meant to be a minimal outline of the}
{code necessary to transmit a file using the YMODEM protocol.}

interface

uses
  wsc, xydrv, Display,
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Menus, StdCtrls;
const
  ONE_K = 0;
  YMODEM = 1;
type
  Tyms = class(TForm)
    MainMenu1: TMainMenu;
    mSend: TMenuItem;
    mExit: TMenuItem;
    eResult: TMemo;
    procedure mExitClick(Sender: TObject);
    procedure mSendClick(Sender: TObject);
    procedure create(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    Port : Integer;
    BaudRate : LongInt;
    FileName : String;
  end;

var
  yms: Tyms;

implementation

{$R *.DFM}

procedure Tyms.mExitClick(Sender: TObject);
begin
  Application.Terminate
end;

procedure Tyms.mSendClick(Sender: TObject);
var
  I, Code : Integer;
  FilePtr : PChar;
  MsgPtr  : PChar;
  Packet  : Integer;
  LastPacket : Integer;
begin
  (* initialize WSC *)
  Code := SioReset(Port,2048,2048);
  if Code < 0 then
    begin
      DisplayLine(yms.eResult,Format('Error %d: Cannot reset port',[Code]));
      DisplayError(yms.eResult, Code);
      exit
    end;
  (* set parameters *)
  SioBaud(Port, BaudRate);
  SioParms(Port, WSC_NoParity, WSC_OneStopBit, WSC_WordLength8);
  SioDTR(Port,'S');
  SioRTS(Port,'S');
  (* set hardware flow control *)
  SioFlow(Port, 'H');
  (* acquire xyDriver *)
  xyAcquire(Port,Port);
  (* XM Send *)
  GetMem(FilePtr,50);
  StrPCopy(FilePtr,FileName);
  DisplayLine(yms.eResult,'Starting YMODEM send');
  Code := xyStartTx(Port,FilePtr,ONE_K,YMODEM);
  FreeMem(FilePtr,50);
  (* check return code *)
  if Code < 0 then
    begin
      DisplayLine(yms.eResult,Format('Error %d: YMODEM send fails',[Code]));
      exit
    end;
  (* allocate memory for XY driver messages *)
  GetMem(MsgPtr,90);
  (* ready to go *)
  SCREEN.Cursor := crHourGlass;
  mSend.Enabled := false;
  (* send file *)
  while true do
    begin
      (* run the XY driver a few times *)
      for I:= 1 To 4 do Code := xyDriver(Port);
      while true do
        begin
          (* get next message from XY driver *)
          if xyGetMessage(Port,MsgPtr,90) <> 0 then
            DisplayLine(yms.eResult,StrPas(MsgPtr))
          else break;
        end;
      (* is XY driver done ? *)
      if Code = XY_IDLE then
        begin
          (* XY driver is idle *)
          Code := xyGetParameter(Port,XY_GET_ERROR_CODE);
          if Code <> 0 then
            DisplayLine(yms.eResult,Format('XY file transfer fails (%d)',[Code]))
          else DisplayLine(yms.eResult,'File transfer complete');
          (* all done *)
          FreeMem(MsgPtr,90);
          xyRelease;
          SioDone(Port);
          mSend.Enabled := true;
          SCREEN.Cursor := crDefault;
          exit
        end
      else
        begin
          (* XY driver is running *)
          Packet := xyGetParameter(Port,XY_GET_PACKET);
          if Packet <> LastPacket then
            begin
              DisplayLine(yms.eResult,Format('Packet %d',[Packet]) );
              LastPacket := Packet
            end
        end
    end
end;

procedure Tyms.create(Sender: TObject);
begin
  (* PROGRAMMER: Edit next 3 lines to suite *)
  Port := COM2;
  BaudRate := 19200;
  FileName := 'file.bin';
  (* END-PROGRAMMER *)
  DisplayLine(yms.eResult,Format('Port : COM%1d',[1+Port]));
  DisplayLine(yms.eResult,Format('Baud : %d',[BaudRate]));
  DisplayLine(yms.eResult,Format('File : %s',[FileName]));
  DisplayLine(yms.eResult,'Push "Send" above to transmit file')
end;

end.
